<?php
/**
 * Client Handler.
 *
 * @package MainWP/Dashboard
 */

namespace MainWP\Dashboard;

/**
 * Class MainWP_Client_Handler
 *
 * @package MainWP\Dashboard
 *
 * @uses \MainWP\Dashboard\MainWP_Client_Handler
 */
class MainWP_Client_Handler {

	/**
	 * Private static variable to hold the single instance of the class.
	 *
	 * @static
	 *
	 * @var mixed Default null
	 */
	private static $instance = null;

	/**
	 * Method instance()
	 *
	 * Create a public static instance.
	 *
	 * @static
	 * @return MainWP_Client_Handler
	 */
	public static function instance() {
		if ( null == self::$instance ) {
			self::$instance = new self();
		}
		return self::$instance;
	}


	/**
	 * Get default client fields.
	 *
	 * Get default client fields.
	 */
	public static function get_default_client_fields() {

		return array(
			'client.name'              => array(
				'title'    => __( 'Client Name (Required)', 'mainwp' ),
				'desc'     => __( 'Displays the Client name', 'mainwp' ),
				'db_field' => 'name',
			),
			'client.email'             => array(
				'title'    => __( 'Client email', 'mainwp' ),
				'desc'     => __( 'Displays the client email', 'mainwp' ),
				'db_field' => 'client_email',
			),
			'client.phone'             => array(
				'title'    => __( 'Client phone', 'mainwp' ),
				'desc'     => __( 'Displays the client phone', 'mainwp' ),
				'db_field' => 'client_phone',
			),
			'client.facebook'          => array(
				'title'    => __( 'Client Facebook', 'mainwp' ),
				'desc'     => __( 'Displays the client Facebook', 'mainwp' ),
				'db_field' => 'client_facebook',
			),
			'client.twitter'           => array(
				'title'    => __( 'Client Twitter', 'mainwp' ),
				'desc'     => __( 'Displays the client Twitter', 'mainwp' ),
				'db_field' => 'client_twitter',
			),
			'client.instagram'         => array(
				'title'    => __( 'Client Instagram', 'mainwp' ),
				'desc'     => __( 'Displays the client Instagram', 'mainwp' ),
				'db_field' => 'client_instagram',
			),
			'client.linkedin'          => array(
				'title'    => __( 'Client LinkedIn', 'mainwp' ),
				'desc'     => __( 'Displays the client LinkedIn', 'mainwp' ),
				'db_field' => 'client_linkedin',
			),
			'client.contact.address.1' => array(
				'title'    => __( 'Client address 1', 'mainwp' ),
				'desc'     => __( 'Displays the client contact address 1', 'mainwp' ),
				'db_field' => 'address_1',
			),
			'client.contact.address.2' => array(
				'title'    => __( 'Client address 2', 'mainwp' ),
				'desc'     => __( 'Displays the client contact address 2', 'mainwp' ),
				'db_field' => 'address_2',
			),
			'client.city'              => array(
				'title'    => __( 'Client city', 'mainwp' ),
				'desc'     => __( 'Displays the client city', 'mainwp' ),
				'db_field' => 'city',
			),
			'client.state'             => array(
				'title'    => __( 'Client state', 'mainwp' ),
				'desc'     => __( 'Displays the client state', 'mainwp' ),
				'db_field' => 'state',
			),
			'client.zip'               => array(
				'title'    => __( 'Client ZIP', 'mainwp' ),
				'desc'     => __( 'Displays the client ZIP code', 'mainwp' ),
				'db_field' => 'zip',
			),
			'client.country'           => array(
				'title'    => __( 'Client country', 'mainwp' ),
				'desc'     => __( 'Displays the client country', 'mainwp' ),
				'db_field' => 'country',
			),
			'client.note'              => array(
				'title'    => __( 'Client notes', 'mainwp' ),
				'desc'     => __( 'Displays the client note', 'mainwp' ),
				'db_field' => 'note',
			),
			'client.suspended'         => array(
				'title'    => __( 'Client status', 'mainwp' ),
				'desc'     => __( 'Displays the client suspended state', 'mainwp' ),
				'db_field' => 'suspended',
			),
		);
	}


	/**
	 * Get default contact fields.
	 *
	 * Get default contact fields.
	 */
	public static function get_default_contact_fields() {
		return array(
			'client.contact.name' => array(
				'title'    => __( 'Contact name (Required)', 'mainwp' ),
				'desc'     => __( 'Displays the client contact name', 'mainwp' ),
				'db_field' => 'contact_name',
			),
			'contact.role'        => array(
				'title'    => __( 'Contact role', 'mainwp' ),
				'desc'     => __( 'Displays the contact role', 'mainwp' ),
				'db_field' => 'contact_role',
			),
			'contact.email'       => array(
				'title'    => __( 'Contact email (Required)', 'mainwp' ),
				'desc'     => __( 'Displays the contact email', 'mainwp' ),
				'db_field' => 'contact_email',
			),
			'contact.phone'       => array(
				'title'    => __( 'Contact phone', 'mainwp' ),
				'desc'     => __( 'Displays the contact phone', 'mainwp' ),
				'db_field' => 'contact_phone',
			),
			'contact.facebook'    => array(
				'title'    => __( 'Contact Facebook', 'mainwp' ),
				'desc'     => __( 'Displays the contact Facebook', 'mainwp' ),
				'db_field' => 'facebook',
			),
			'contact.twitter'     => array(
				'title'    => __( 'Contact Twitter', 'mainwp' ),
				'desc'     => __( 'Displays the contact Twitter', 'mainwp' ),
				'db_field' => 'twitter',
			),
			'contact.instagram'   => array(
				'title'    => __( 'Contact Instagram', 'mainwp' ),
				'desc'     => __( 'Displays the contact Instagram', 'mainwp' ),
				'db_field' => 'instagram',
			),
			'contact.linkedin'    => array(
				'title'    => __( 'Contact LinkedIn', 'mainwp' ),
				'desc'     => __( 'Displays the contact LinkedIn', 'mainwp' ),
				'db_field' => 'linkedin',
			),
		);
	}


	/**
	 * Method get_compatible_tokens().
	 *
	 * Get compatible tokens.
	 */
	public static function get_compatible_tokens() {
		return array(
			'client.contact.name' => 'client.contact.name',
			'client.email'        => 'contact.email',
			'client.phone'        => 'contact.phone',
		);
	}
	/**
	 * Method rest_api_add_client().
	 *
	 * Rest API add client.
	 *
	 * @param array $data fields array.
	 * @param bool  $edit Is edit.
	 *
	 *  $data fields.
	 *  'email'.
	 *  'name'.
	 *  'facebook'.
	 *  'twitter'.
	 *  'instagram'.
	 *  'address_1'.
	 *  'address_2'.
	 *  'city'.
	 *  'zip'.
	 *  'state'.
	 *  'note'.
	 *  'selected_sites'.
	 *  'client_id'. - to edit client.
	 *
	 * @return mixed Results.
	 */
	public static function rest_api_add_client( $data, $edit = false ) { // phpcs:ignore -- comlex function. Current complexity is the only way to achieve desired results, pull request solutions appreciated.  

		$params = array();

		if ( $edit ) {

			$client_id = isset( $data['client_id'] ) ? intval( $data['client_id'] ) : 0;

			if ( empty( $client_id ) ) { // edit client.
				return array(
					'error' => __( 'Client ID field is required! Please enter a Client ID.', 'mainwp' ),
				);
			}

			$params['client_id'] = $client_id;

			if ( isset( $data['name'] ) ) {
				$params['name'] = sanitize_text_field( wp_unslash( $data['name'] ) );
			}

			if ( isset( $data['address_1'] ) ) {
				$params['address_1'] = sanitize_text_field( wp_unslash( $data['address_1'] ) );
			}
			if ( isset( $data['address_2'] ) ) {
				$params['address_2'] = sanitize_text_field( wp_unslash( $data['address_2'] ) );
			}
			if ( isset( $data['city'] ) ) {
				$params['city'] = sanitize_text_field( wp_unslash( $data['city'] ) );
			}
			if ( isset( $data['zip'] ) ) {
				$params['zip'] = sanitize_text_field( wp_unslash( $data['zip'] ) );
			}
			if ( isset( $data['state'] ) ) {
				$params['state'] = sanitize_text_field( wp_unslash( $data['state'] ) );
			}

			if ( isset( $data['country'] ) ) {
				$params['country'] = sanitize_text_field( wp_unslash( $data['country'] ) );
			}

			if ( isset( $data['note'] ) ) {
				$params['note'] = sanitize_text_field( wp_unslash( $data['note'] ) );
			}

			MainWP_DB_Client::instance()->update_client( $params );

			if ( isset( $data['selected_sites'] ) && is_array( $data['selected_sites'] ) ) {
				$selected_sites = array_map( 'sanitize_text_field', wp_unslash( $data['selected_sites'] ) );
				MainWP_DB_Client::instance()->update_selected_sites_for_client( $client_id, $selected_sites );
			}

			if ( isset( $data['custom_fields'] ) && is_array( $data['custom_fields'] ) ) {
				$custom_fields = array_map( 'sanitize_text_field', wp_unslash( $data['custom_fields'] ) );
				foreach ( $custom_fields as $field ) {
					$fie_name  = isset( $field['field_name'] ) ? sanitize_text_field( wp_unslash( $data['field_name'] ) ) : '';
					$fie_value = isset( $field['field_value'] ) ? sanitize_text_field( wp_unslash( $data['field_value'] ) ) : '';
					$fie_desc  = isset( $field['field_desc'] ) ? sanitize_text_field( wp_unslash( $data['field_desc'] ) ) : '';

					if ( ! empty( $fie_name ) && ! empty( $fie_value ) ) {
						$get_gen_field = MainWP_DB_Client::instance()->get_client_fields_by( 'field_name', $fie_name, 0 );
						if ( $get_gen_field ) { // it is general field.
							if ( ! empty( $fie_desc ) ) {
								MainWP_DB_Client::instance()->update_client_field(
									$get_gen_field->field_id,
									array(
										'field_desc' => $fie_desc,
									)
								);
							}
							MainWP_DB_Client::instance()->update_client_field_value( $get_gen_field->field_id, $fie_value, $client_id ); // add or update general field value for the client.
						} else {
							$indi_gen_field = MainWP_DB_Client::instance()->get_client_fields_by( 'field_name', $fie_name, $client_id );
							if ( $indi_gen_field ) { // it is individual field.

								if ( ! empty( $fie_desc ) ) {
									MainWP_DB_Client::instance()->update_client_field(
										$indi_gen_field->field_id,
										array(
											'field_desc' => $fie_desc,
											'client_id'  => $client_id,
										)
									);
								}
								MainWP_DB_Client::instance()->update_client_field_value( $indi_gen_field->field_id, $fie_value, $client_id ); // add or update individual field value for the client.
							}
						}
					}
				}
			}

			return array(
				'success'  => true,
				'clientid' => $client_id,
			);

		} else {

			$params['name'] = isset( $data['name'] ) ? sanitize_text_field( wp_unslash( $data['name'] ) ) : '';

			if ( empty( $params['name'] ) ) {
				return array(
					'error' => __( 'Client name field is required! Please enter a Client name.', 'mainwp' ),
				);
			}

			$params['address_1'] = isset( $data['address_1'] ) ? sanitize_text_field( wp_unslash( $data['address_1'] ) ) : '';
			$params['address_2'] = isset( $data['address_2'] ) ? sanitize_text_field( wp_unslash( $data['address_2'] ) ) : '';
			$params['city']      = isset( $data['city'] ) ? sanitize_text_field( wp_unslash( $data['city'] ) ) : '';
			$params['zip']       = isset( $data['zip'] ) ? sanitize_text_field( wp_unslash( $data['zip'] ) ) : '';
			$params['state']     = isset( $data['state'] ) ? sanitize_text_field( wp_unslash( $data['state'] ) ) : '';
			$params['country']   = isset( $data['country'] ) ? sanitize_text_field( wp_unslash( $data['country'] ) ) : '';
			$params['note']      = isset( $data['note'] ) ? sanitize_text_field( wp_unslash( $data['note'] ) ) : '';

			$inserted = MainWP_DB_Client::instance()->update_client( $params );

			if ( empty( $inserted ) ) {
				return array(
					'error' => __( 'Undefined error. Please try again.', 'mainwp' ),
				);
			}

			$selected_sites = ( isset( $data['selected_sites'] ) && is_array( $data['selected_sites'] ) ) ? array_map( 'sanitize_text_field', wp_unslash( $data['selected_sites'] ) ) : array();

			MainWP_DB_Client::instance()->update_selected_sites_for_client( $inserted->client_id, $selected_sites );

			return array(
				'success'  => true,
				'clientid' => $inserted->client_id,
			);
		}
	}


	/**
	 * Method get_website_client_tokens_data()
	 *
	 * Get website client tokens.
	 *
	 * @param int $websiteid Website ID.
	 *
	 * @return mixed $result Result of tokens.
	 */
	public function get_website_client_tokens_data( $websiteid = false ) { // phpcs:ignore -- comlex function. Current complexity is the only way to achieve desired results, pull request solutions appreciated. 
		if ( empty( $websiteid ) ) {
			return false;
		}
		$website = MainWP_DB::instance()->get_website_by_id( $websiteid );
		if ( empty( $website ) ) {
			return false;
		}

		$client_tokens = array();

		$client_tokens['client.site.url']  = $website->url;
		$client_tokens['client.site.name'] = $website->name;

		// init values.
		$default_client_fields = self::get_default_client_fields();
		foreach ( $default_client_fields as $tok_name => $field ) {
			$client_tokens[ $tok_name ] = '';  // the tokens with empty value will be removed from the report.
		}
		$contact_fields = self::get_default_contact_fields();
		if ( ! empty( $contact_fields ) && is_array( $contact_fields ) ) {
			foreach ( $contact_fields as $tok_name => $field ) {
				$client_tokens[ $tok_name ] = ''; // the tokens with empty value will be removed from the report.
			}
		}

		$custom_tokens_fields = MainWP_DB_Client::instance()->get_client_fields( true, $website->client_id );
		if ( ! empty( $custom_tokens_fields ) && is_array( $custom_tokens_fields ) ) {
			foreach ( $custom_tokens_fields as $fields ) {
				$client_tokens[ $fields->field_name ] = ''; // the tokens with empty value will be removed from the report.
			}
		}

		if ( empty( $website->client_id ) ) {
			return $client_tokens;
		}

		$clientid = $website->client_id;

		$client_info = MainWP_DB_Client::instance()->get_wp_client_by( 'client_id', $clientid );

		if ( empty( $client_info ) ) {
			return $client_tokens;
		}

		if ( ! empty( $custom_tokens_fields ) && is_array( $custom_tokens_fields ) ) {
			foreach ( $custom_tokens_fields as $fields ) {
				$client_tokens[ $fields->field_name ] = $fields->field_value;
			}
		}

		foreach ( $default_client_fields as $tok_name => $field ) {
			$db_field = $field['db_field'];
			if ( property_exists( $client_info, $db_field ) ) {
				$client_tokens[ $tok_name ] = $client_info->{$db_field};
			}
		}

		if ( ! empty( $contact_fields ) && is_array( $contact_fields ) ) {
			foreach ( $contact_fields as $tok_name => $field ) {
				$db_field = isset( $field['db_field'] ) ? $field['db_field'] : '';
				$val      = '' != $db_field && property_exists( $client_info, $db_field ) ? $client_info->{$db_field} : false;
				if ( false !== $val ) {
					$client_tokens[ $tok_name ] = $val;
				}
			}
		}
		$client_tokens = apply_filters( 'mainwp_clients_website_client_tokens', $client_tokens, $websiteid, $clientid );
		return $client_tokens;
	}

	/**
	 * Method get_favico_url()
	 *
	 * Get Child Site favicon URL.
	 *
	 * @param string $image_path Client image url path.
	 *
	 * @return mixed $full_url Full image URL.
	 */
	public static function get_client_image_url( $image_path ) {
		$full_url = '';
		if ( ! empty( $image_path ) ) {
			$dirs = MainWP_System_Utility::get_mainwp_dir( 'client-images', true );
			if ( file_exists( $dirs[0] . $image_path ) ) {
				$full_url = $dirs[1] . $image_path;
			} else {
				$full_url = '';
			}
		}
		return $full_url;
	}
}
